from constants import *
import os
import logging

logger = logging.getLogger(APP_NAME)

class Config:
    def __init__(self, path):
        self._config_path = path
        self.set_state(CLOSED)

    def __setattr__(self, name, value):
        if name[0] != '_':
            self.set_state(UNSAVED)
        self.__dict__[name] = value

    def __getattr__(self, name):
        if name[0] == '_':
            raise AttributeError('Object instance has no attribute %r' % name)
        return None

    def get_state(self):
        return self._state

    def set_state(self, state):
        self._state = state

    def read(self):
        logger.debug('Reading config.')

        with open(self._config_path) as config:
            for line in config:
                line = line.rstrip('\n')

                if line.strip() == '':
                    continue

                elif line[0] == '#':
                    continue

                elif '=' in line:
                    line_split = line.split('=')

                    try:
                        name = line_split[0]
                        value = '='.join(line_split[1:])

                        # If option name ends with s assume list
                        if name[-1] == 's' and name[-2:] != 'ss':
                            value = value.split(',')
                            value[:] = [x for x in value if x != '']

                    except ValueError:
                        logger.warning('Ignoring invalid line. %r' % {
                            'line': line,
                        })
                        continue
                    setattr(self, name, value)

                else:
                    logger.warning('Ignoring invalid line. %r' % {
                        'line': line,
                    })

        self.set_state(SAVED)

    def write(self):
        logger.debug('Writing config.')

        with open(self._config_path, 'w') as config:
            for name in sorted(vars(self).keys()):
                value = vars(self)[name]

                if name[0] == '_':
                    continue
                # If option name ends with s assume list
                elif name[-1] == 's' and name[-2:] != 'ss':
                    value = ','.join(value)
                elif value is None:
                    continue

                config.write('%s=%s\n' % (name, value))

        self.set_state(SAVED)



from constants import *
from task import Task
from event import Event
import os
import time
import shlex
import logging
import shutil
import subprocess

logger = logging.getLogger(APP_NAME)

class MoveVolume(Task):
    def __init__(self, *kargs, **kwargs):
        Task.__init__(self, *kargs, **kwargs)
        self.type = MOVE_VOLUME

    def _get_mount(self, path):
        output = subprocess.check_output(['df', '-P', path])
        return output.strip().split(' ')[-1]

    def _abort_process(self, process):
        for i in xrange(10):
            process.terminate()
            logger.debug('Terminating volume move process. %r' % {
                'volume_id': self.volume_id,
                'task_id': self.id,
                'pid': process.pid,
            })
            time.sleep(1)
            if process.poll() is not None:
                break

        if process.poll() is None:
            for i in xrange(30):
                process.kill()
                logger.debug('Killing volume move process. %r' % {
                    'volume_id': self.volume_id,
                    'task_id': self.id,
                    'pid': process.pid,
                })
                time.sleep(0.5)
                if process.poll() is not None:
                    break

        if process.poll() is None:
            logger.error('Failed to abort volume move process. %r' % {
                'volume_id': self.volume_id,
                'task_id': self.id,
                'pid': process.pid,
            })

        logger.warning('Volume move aborted. %r' % {
            'volume_id': self.volume_id,
            'task_id': self.id,
        })

        self.remove_snapshot()
        self.aborted()

    def run(self):
        source_path = self.volume.orig_path
        destination_path = self.volume.path

        if self._get_mount(source_path) == self._get_mount(destination_path):
            logger.info('Moving volume. %r' % {
                'volume_id': self.volume_id,
            })

            args = ['find', source_path, '-mindepth', '1', '-maxdepth', '1',
                '-exec', 'mv', '-t', destination_path, '--', '{}', ';']

            process = subprocess.Popen(args)
            return_code = None

            while True:
                poll = process.poll()
                if poll is not None:
                    return_code = poll
                    break
                if self.state == ABORTING:
                    self._abort_process(process)
                    return
                time.sleep(0.5)

        else:
            logger.info('Copying volume. %r' % {
                'volume_id': self.volume_id,
            })

            log_path = os.path.join(source_path, LOG_DIR,
                'copy_volume_%s.log' % int(time.time()))

            rsync_source_path = source_path
            if rsync_source_path[-1] != os.sep:
                rsync_source_path += os.sep

            rsync_destination_path = destination_path
            if rsync_destination_path[-1] != os.sep:
                rsync_destination_path += os.sep

            args = ['rsync', '--archive', '--hard-links', '--acls',
                '--quiet', '--xattrs',  '--progress', '--super',
                '--log-file-format=%o \"%f\" %l', '--log-file=%s' % log_path,
                rsync_source_path, rsync_destination_path]

            process = subprocess.Popen(args)
            return_code = None

            while True:
                poll = process.poll()
                if poll is not None:
                    return_code = poll
                    break
                if self.state == ABORTING:
                    self._abort_process(process)
                    return
                time.sleep(0.5)

            logger.debug('Removing previous copy of volume. %r' % {
                'volume_id': self.volume_id,
                'task_id': self.id,
            })

            for name in os.listdir(source_path):
                path = os.path.join(source_path, name)
                try:
                    if os.path.isfile(path):
                        os.remove(path)
                    else:
                        shutil.rmtree(path)
                except Exception:
                    logger.debug('Failed to delete path while removing ' +
                        'previous volume files. %r' % {
                            'volume_id': self.volume_id,
                            'task_id': self.id,
                        })

        if return_code != 0:
            raise OSError('Volume move failed, command ' + \
                'returned non-zero exit status. %r' % {
                    'volume_id': self.volume_id,
                    'task_id': self.id,
                })
        else:
            logger.debug('Volume move complete, committing config. %r' % {
                'volume_id': self.volume_id,
                'task_id': self.id,
            })
            self.volume.client.commit()

        Event(type=VOLUMES_UPDATED)



from constants import *
from task import Task
from event import Event
import os
import time
import shlex
import logging
import shutil
import subprocess

logger = logging.getLogger(APP_NAME)

class RemoveSnapshot(Task):
    def __init__(self, *kargs, **kwargs):
        Task.__init__(self, *kargs, **kwargs)
        self.type = REMOVE_SNAPSHOT

    def _abort_process(self, process):
        for i in xrange(10):
            process.terminate()
            logger.debug('Terminating snapshot remove process. %r' % {
                'volume_id': self.volume_id,
                'snapshot_id': self.snapshot_id,
                'task_id': self.id,
                'pid': process.pid,
            })
            time.sleep(1)
            if process.poll() is not None:
                break

        if process.poll() is None:
            for i in xrange(30):
                process.kill()
                logger.debug('Killing snapshot remove process. %r' % {
                    'volume_id': self.volume_id,
                    'snapshot_id': self.snapshot_id,
                    'task_id': self.id,
                    'pid': process.pid,
                })
                time.sleep(0.5)
                if process.poll() is not None:
                    break

        if process.poll() is None:
            logger.error('Failed to abort snapshot remove process. %r' % {
                'volume_id': self.volume_id,
                'snapshot_id': self.snapshot_id,
                'task_id': self.id,
                'pid': process.pid,
            })

        logger.warning('Snapshot remove aborted. %r' % {
            'volume_id': self.volume_id,
            'snapshot_id': self.snapshot_id,
            'task_id': self.id,
        })

        self.aborted()

    def run(self, keep_log=False):
        if not keep_log:
            logger.debug('Removing snapshot log file. %r' % {
                'volume_id': self.volume_id,
                'snapshot_id': self.snapshot_id,
                'task_id': self.id,
            })

            if os.path.isfile(self.snapshot.log_path):
                os.remove(self.snapshot.log_path)

        if not os.path.isdir(self.snapshot.path):
            return

        snapshot_path_temp = '%s.removing' % self.snapshot.path
        snapshot_path_failed = '%s.failed' % self.snapshot.path

        try:
            os.rename(self.snapshot.path, snapshot_path_temp)
        except OSError:
            logging.error('Snapshot remove failed, ' + \
                'unable to rename snapshot temp directory. %r' % {
                    'volume_id': self.volume_id,
                    'snapshot_id': self.snapshot_id,
                    'task_id': self.id,
                })
            raise

        logger.debug('Removing snapshot directory. %r' % {
            'volume_id': self.volume_id,
            'snapshot_id': self.snapshot_id,
            'task_id': self.id,
        })

        args = ['rm', '-rf', snapshot_path_temp]

        process = subprocess.Popen(args)
        return_code = None

        while True:
            poll = process.poll()
            if poll is not None:
                return_code = poll
                break
            if self.state == ABORTING:
                # Attempt to set snapshot as failed
                try:
                    os.rename(snapshot_path_temp, snapshot_path_failed)
                except OSError:
                    pass

                self._abort_process(process)
                return
            time.sleep(0.5)

        time.sleep(5)

        Event(volume_id=self.volume_id, type=SNAPSHOTS_UPDATED)
